local _, unitClass = UnitClass("player")

local DoaschHero = CreateFrame("Frame", "TemplateFrame", UIParent)

local doaschHeroVersion = "DoaschHero 0.6"

local heroismIconPath
local heroismIconCDPath = "Interface/Icons/Ability_Creature_Cursed_02"
local spellId = nil
local heroRange = 100

-- Slash command
SLASH_DOASCHHERO1 = "/doaschhero"
function SlashCmdList.DOASCHHERO(msg, editbox)
    print(doaschHeroVersion)
end

-- Set the icons and skills depending on the class
if unitClass == "SHAMAN" then
    if UnitFactionGroup("player") == "Alliance" then
        heroismIconPath = "Interface/Icons/Ability_Shaman_Heroism"
        spellId = 32182
    elseif UnitFactionGroup("player") == "Horde" then
        heroismIconPath = "Interface/Icons/Spell_Nature_Bloodlust"
        spellId = 2825
    end
elseif unitClass == "MAGE" then
    heroismIconPath = "Interface/Icons/Ability_Mage_TimeWarp"
    spellId = 80353
end

-- Checks if the unit has a heorism debuff
function HasHeroismDebuff(unit)
    if UnitDebuff(unit, "Exhaustion") or UnitDebuff(unit, "Temporal Displacement") or UnitDebuff(unit, "Sated") then
        return true
    end
    
    return false
end

function DoaschHero:CreateLayout()
	DoaschHeroHeader = _G["DoaschHeroFrame"]
    
    DoaschHeroButton = CreateFrame("Button", "DoaschHeroButton", DoaschHeroHeader, "SecureActionButtonTemplate")
    DoaschHeroButton:SetPoint("CENTER",0,0)
    DoaschHeroButton:SetWidth(40)
    DoaschHeroButton:SetHeight(40)
    DoaschHeroButton:SetNormalTexture(heroismIconPath)
    DoaschHeroButton:SetDisabledTexture(heroismIconCDPath)
    DoaschHeroButton:RegisterForClicks("LeftButtonUp")
    
    local spellName = GetSpellInfo(spellId)
    DoaschHeroButton:SetAttribute("type", "spell")
    DoaschHeroButton:SetAttribute("spell", spellName)
end
    
-- Setup the UI
DoaschHero:CreateLayout()

if spellId ~= nil then
    -- Okay, we could use hero (based on class)

    DoaschHero:SetScript("OnUpdate", function(self, elapsed)
        -- Can we really use it aka. did we already learn it?
        if IsPlayerSpell(spellId) then
            -- Display frame as we could have gained a level up
            DoaschHeroHeader:Show()
           
            -- Can we even cast hero (e.g. it is not on CD)
            if GetSpellCooldown(spellId) ~= 0 then -- We have CD -> No cast possib.
                DoaschHeroHeader:SetBackdropColor(1, 0, 0, 1)
                DoaschHeroButton:Disable()
            else
                -- Check how many players do have the debuff
                local numGM = GetNumGroupMembers()
                local numDebuffs = 0
               
                -- numGM is 0 if there the player is in no group
                if numGM == 0 then
                    numGM = numGM + 1
                else
                    -- Check for debuffs of players
                    for i = 0, numGM do
                        if HasHeroismDebuff("raid" .. i) or HasHeroismDebuff("party" .. i) then
                            numDebuffs = numDebuffs + 1
                        end
                    end
                end
               
                if HasHeroismDebuff("player") then
                    numDebuffs = numDebuffs + 1
                end

                if numDebuffs == numGM then
                    -- Everybody has the Debuff -> unclickable
                    DoaschHeroHeader:SetBackdropColor(1, 0, 0, 1)
                    DoaschHeroButton:Disable()
                elseif numDebuffs < numGM and numDebuffs > 0 then
                    -- Not everybody has the debuff, make it clickable
                    DoaschHeroHeader:SetBackdropColor(1, 1, 0, 1)
                    DoaschHeroButton:Enable()
                else
                    -- No one has the debuff -> we are good to go
                    DoaschHeroHeader:SetBackdropColor(0, 1, 0, 1)
                    DoaschHeroButton:Enable()
                end
           end
        else
           -- Our class can use hero but we are too low level
           DoaschHeroHeader:Hide()
        end
    end)
else
    -- No hero class / level not high enough -> hide UI
    DoaschHeroHeader:Hide()
end
